//
//  MovieWrapper.m
//  CocoaQuicktime
//
//  Created by fred on Wed Oct 18 2000.
//  Copyright (c) 2000 __CompanyName__. All rights reserved.
//

#import "MovieWrapper.h"

@interface MovieWrapper(Private)
//	Creatre an image rep with frame at aTimeValue
- (NSImageRep *)_imageRepAtTimeValue:(TimeValue)aTimeValue;
@end

@implementation MovieWrapper(Private)

- (NSImageRep *)_imageRepAtTimeValue:(TimeValue)aTimeValue
{	SignedByte theState;
	PicHandle thePicHandle;
	NSData *thePictData;
	NSImageRep *theImageRep;
	Size theSize;

	thePicHandle = GetMoviePict( itsMovie,  aTimeValue );
	NSAssert( thePicHandle, @"GetMoviePict" );

	theSize = GetHandleSize( (Handle)thePicHandle );
	NSAssert( theSize, @"GetHandleSize" );

	theState = HGetState( (Handle)thePicHandle );
	HLock( (Handle)thePicHandle );

	thePictData = [[[NSData dataWithBytes:*thePicHandle length:theSize] copy] autorelease];
	theImageRep = [NSPICTImageRep imageRepWithData:thePictData];

	HSetState( (Handle)thePicHandle, theState );
	KillPicture( thePicHandle );

	return theImageRep;
}

@end

@implementation MovieWrapper

+ (MovieWrapper *)movieWrapperFromPath:(NSString *)aPath
{	return [[[self alloc] initFromPath:aPath] autorelease];
}

- initWithMovie:(Movie)aMovie
{	[super init];
	itsMovie = aMovie;
	return self;
}

- initFromFSSpec:(FSSpec)aFSSpec
{	OSErr theError;
	short theMovieResID = 0;
	Str255 theMovieName;
	Boolean thefWasChanged;
	Movie theMovie;

	theError = OpenMovieFile ( &aFSSpec, &itsMovieResFile, 0);
	NSAssert( !theError, @"OpenMovieFile" );

	theError = NewMovieFromFile( &theMovie, itsMovieResFile, &theMovieResID, theMovieName, newMovieActive, &thefWasChanged);
	NSAssert( !theError, @"NewMovieFromFile" );

	return [self initWithMovie:theMovie];
}

- initFromPath:(NSString *)aPath
{	return [self initFromFSSpec:[aPath fsSpec]];
}

- (void)dealloc
{	CloseMovieFile( itsMovieResFile );
//	#### We should release all QT resources in dealloc
//	I bet the Movie should be released somehow (Maybe I should read the docs :-) )
	[super dealloc];
}

- (TimeValue)duration
{	return GetMovieDuration( itsMovie );
}

- (NSImage *)imageAtTimeValue:(TimeValue)aTimeValue
{	NSImage *theImage = [[[NSImage alloc] init] autorelease];
	[theImage addRepresentation:[self _imageRepAtTimeValue:aTimeValue]];
	return theImage;
}

@end


//	Almost verbatim from a mail from bwebster@rice.edu
@implementation NSString(Carbon_Additions)

-(FSSpec)fsSpec
 {	CFURLRef url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, (CFStringRef)self, kCFURLPOSIXPathStyle, false);
 	FSRef ref;
 	FSSpec spec;
 	OSErr err = noErr;
 	
 	if(!CFURLGetFSRef(url, &ref))
 		NSLog(@"error %d occured when creating the FSRef", err); 	else
 	{
 		err = FSGetCatalogInfo(&ref, kFSCatInfoNone, NULL, NULL, &spec, NULL);
 		if(err != noErr)
 			NSLog(@"error %d occured when getting the FSSpec", err);
 	}
 	CFRelease(url);
 	return spec;
 }

@end

